==> jscriptbasic-1.0.5-sources.jar: com/scriptbasic/memory/BasicVariableRegistry.java <==
package com.scriptbasic.memory;

import com.scriptbasic.interfaces.BasicRuntimeException;
import com.scriptbasic.interfaces.ExecutionException;

import java.util.HashMap;
import java.util.Map;

/**
 * @author Peter Verhas
 * date June 25, 2012
 */
class BasicVariableRegistry extends BasicVariableMap {
    private final Map<String, VariableType> registrationMap = new HashMap<>();
    private BasicVariableMap that;

    BasicVariableRegistry(final BasicVariableMap that) {
        super(that);
        this.that = that;
    }

    /**
     * Assert that a variable is not registered to be local and global at the
     * same time. The implementation is lenient to allow double registration of
     * local and global but if ever a variable was registered to be local then
     * it has to remain that and the same holds for global.
     *
     * @param variableName
     * @param type         either KNOWN_LOCAL or KNOWN_GLOBAL. The type that we intend
     *                     the variable to be registered. So it should be KNOWN_LOCAL
     *                     when the caller wants to register a variable as local.
     * @throws ExecutionException when the variable was already registered with a different
     *                            type
     */
    private void assertNoOverregistration(final String variableName, final VariableType type)
            throws ExecutionException {
        that.assertCorrectCasing(variableName);
        final String convertedVariableName = converted(variableName);
        if (registrationMap.containsKey(convertedVariableName)
                && registrationMap.get(convertedVariableName) != type) {
            throw new BasicRuntimeException("Variable '" + variableName
                    + "' can not be local and global at a time.");
        }
    }

    void registerLocal(final String variableName) throws ExecutionException {
        assertNoOverregistration(variableName, VariableType.KNOWN_LOCAL);
        final String convertedVariableName = converted(variableName);
        that.registerVariableCasing(variableName);
        registrationMap.put(convertedVariableName, VariableType.KNOWN_LOCAL);
    }

    void registerGlobal(final String variableName) throws ExecutionException {
        assertNoOverregistration(variableName, VariableType.KNOWN_GLOBAL);
        final String convertedVariableName = converted(variableName);
        that.registerVariableCasing(variableName);
        registrationMap.put(convertedVariableName, VariableType.KNOWN_GLOBAL);
    }

    boolean isGlobal(final String variableName) throws ExecutionException {
        that.assertCorrectCasing(variableName);
        final String convertedVariableName = converted(variableName);
        return registrationMap.get(convertedVariableName) == VariableType.KNOWN_GLOBAL;
    }

    boolean isLocal(final String variableName) throws ExecutionException {
        that.assertCorrectCasing(variableName);
        final String convertedVariableName = converted(variableName);
        return registrationMap.get(convertedVariableName) == VariableType.KNOWN_LOCAL;
    }

    private enum VariableType {
        KNOWN_LOCAL, KNOWN_GLOBAL
    }
}

==> jscriptbasic-2.0.0-sources.jar: com/scriptbasic/memory/BasicVariableRegistry.java <==
package com.scriptbasic.memory;

import com.scriptbasic.api.ScriptBasicException;
import com.scriptbasic.interfaces.BasicRuntimeException;

import java.util.HashMap;
import java.util.Map;

/**
 * @author Peter Verhas
 * date June 25, 2012
 */
class BasicVariableRegistry extends BasicVariableMap {
    private final Map<String, VariableType> registrationMap = new HashMap<>();
    private final BasicVariableMap that;

    BasicVariableRegistry(final BasicVariableMap that) {
        super(that);
        this.that = that;
    }

    /**
     * Assert that a variable is not registered to be local and global at the
     * same time. The implementation is lenient to allow double registration of
     * local and global but if ever a variable was registered to be local then
     * it has to remain that and the same holds for global.
     *
     * @param variableName parameter
     * @param type         either KNOWN_LOCAL or KNOWN_GLOBAL. The type that we intend
     *                     the variable to be registered. So it should be KNOWN_LOCAL
     *                     when the caller wants to register a variable as local.
     * @throws ScriptBasicException when the variable was already registered with a different
     *                              type
     */
    private void assertNoOverregistration(final String variableName, final VariableType type)
            throws ScriptBasicException {
        that.assertCorrectCasing(variableName);
        final var convertedVariableName = converted(variableName);
        if (registrationMap.containsKey(convertedVariableName)
                && registrationMap.get(convertedVariableName) != type) {
            throw new BasicRuntimeException("Variable '" + variableName
                    + "' can not be local and global at a time.");
        }
    }

    void registerLocal(final String variableName) throws ScriptBasicException {
        assertNoOverregistration(variableName, VariableType.KNOWN_LOCAL);
        final var convertedVariableName = converted(variableName);
        that.registerVariableCasing(variableName);
        registrationMap.put(convertedVariableName, VariableType.KNOWN_LOCAL);
    }

    void registerGlobal(final String variableName) throws ScriptBasicException {
        assertNoOverregistration(variableName, VariableType.KNOWN_GLOBAL);
        final var convertedVariableName = converted(variableName);
        that.registerVariableCasing(variableName);
        registrationMap.put(convertedVariableName, VariableType.KNOWN_GLOBAL);
    }

    boolean isGlobal(final String variableName) throws ScriptBasicException {
        that.assertCorrectCasing(variableName);
        final var convertedVariableName = converted(variableName);
        return registrationMap.get(convertedVariableName) == VariableType.KNOWN_GLOBAL;
    }

    boolean isLocal(final String variableName) throws ScriptBasicException {
        that.assertCorrectCasing(variableName);
        final var convertedVariableName = converted(variableName);
        return registrationMap.get(convertedVariableName) == VariableType.KNOWN_LOCAL;
    }

    private enum VariableType {
        KNOWN_LOCAL, KNOWN_GLOBAL
    }
}
